;; photoblogger
;; Copyright (C) 2004 Andy Wingo

;; This program is free software; you can redistribute it and/or    
;; modify it under the terms of the GNU General Public License as   
;; published by the Free Software Foundation; either version 2 of   
;; the License, or (at your option) any later version.              
;;                                                                  
;; This program is distributed in the hope that it will be useful,  
;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
;; GNU General Public License for more details.                     
;;                                                                  
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, contact:
;;
;; Free Software Foundation           Voice:  +1-617-542-5942
;; 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
;; Boston, MA  02111-1307,  USA       gnu@gnu.org

;;; Commentary:
;;
;; Photoblogger uses the same GConf behavior as Owen Taylor's font viewer app: Save
;; the complete app state into GConf. Initialize state from GConf on
;; load, save changes as they occur, but don't monitor changes to the
;; configuration source. This makes it useful to have multiple copies of
;; the app running.
;;
;;; Code:

(define-module (photoblogger)
  :use-module (gnome gtk)
  :use-module (gnome glade)
  :use-module (photoblogger config)
  :export (with-slots <photo> resize-factor resize-pixbuf
           glade-xml main-window tree-view status
           orig-pixbuf file-name base-name extension description))

(define glade-xml (glade-xml-new glade-file))
(define main-window (get-widget glade-xml "window"))
(define tree-view (get-widget glade-xml "treeview"))
(define status (get-widget glade-xml "status"))

;; would be nice to have symbol-macrolet
(define-macro (with-slots obj slot-list . body)
  `(if ,obj
       (let ,(map (lambda (s) `(,s (slot-ref ,obj ',s))) slot-list)
         ,@body)))

;; the 'pixbuf gobject property is the thumbnail
(define-class <photo> (<gtk-image>)
  (orig-pixbuf :init-keyword :orig-pixbuf :accessor orig-pixbuf)
  (file-name :init-keyword :file-name :accessor file-name)
  (base-name :init-keyword :base-name :accessor base-name)
  (extension :init-keyword :extension :accessor extension)
  (description :init-keyword :description :accessor description :init-value #f)
  (already-uploaded :init-keyword :already-uploaded :init-value #f))

(define (resize-factor p x y)
  (let* ((orig-x (get-width p))
         (orig-y (get-height p)))
    (if (> (/ orig-x orig-y) (/ x y))
        (/ x orig-x)
        (/ y orig-y))))

(define (resize-pixbuf p x y)
  (let* ((orig-x (get-width p))
         (orig-y (get-height p))
         (factor (resize-factor p x y))
         (new-x (inexact->exact (* orig-x factor)))
         (new-y (inexact->exact (* orig-y factor)))
         (new (gdk-pixbuf-new 'rgb #f 8 new-x new-y)))
    (gdk-pixbuf-scale
     p new 0 0 new-x new-y 0 0 factor factor 'bilinear)
    new))

(define-method (initialize (obj <photo>) initargs)
  (next-method)
  (with-slots obj (file-name)
    (let ((s (basename file-name)))
      (set! (extension obj)
            (substring s (or (string-rindex s #\.)
                             (string-length s))))
      (set! (base-name obj)
            (substring s 0 (or (string-rindex s #\.)
                               (string-length s)))))
    (set! (orig-pixbuf obj) (gdk-pixbuf-new-from-file file-name))
    (set obj 'pixbuf (resize-pixbuf (orig-pixbuf obj) 64 64))))
