;; photoblogger
;; Copyright (C) 2004 Andy Wingo

;; This program is free software; you can redistribute it and/or    
;; modify it under the terms of the GNU General Public License as   
;; published by the Free Software Foundation; either version 2 of   
;; the License, or (at your option) any later version.              
;;                                                                  
;; This program is distributed in the hope that it will be useful,  
;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
;; GNU General Public License for more details.                     
;;                                                                  
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, contact:
;;
;; Free Software Foundation           Voice:  +1-617-542-5942
;; 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
;; Boston, MA  02111-1307,  USA       gnu@gnu.org

;;; Commentary:
;;
;;; Code:

(define-module (photoblogger dialogs)
  :use-module (gnome gtk)
  :export (question error-message dialog))

(define (add-buttons w button-specs)
  (for-each
   (lambda (spec)
     (let* ((response (genum->value
                       (make <gtk-response-type> :value (car spec))))
            (text (cadr spec))
            (icon (and (caddr spec)
                       (make <gtk-image> :stock (gtk-stock-id (caddr spec)))))
            (align (gtk-alignment-new 0.5 0.5 0 0))
            (hbox (make <gtk-hbox>))
            (button (make <gtk-button>))
            (label (make <gtk-label> :label text :use-underline #t
                         :mnemonic-widget button)))
       (add button align)
       (add align hbox)
       (if icon (pack-start hbox icon #f #f 1))
       (pack-start hbox label #f #f 1)
       (show-all button)
       ;; doesn't work now, as there's no set-flags interface
       ;; (grab-default button) ;; default is the last button
       (grab-focus button) ;; Why doesn't this work? Ah well..
       (add-action-widget w button response)))
   button-specs))

(define (dialog type primary secondary . button-specs)
  "Ask a user a question. Each additional argument should have the form
@var{(response-symbol text-string stock-icon)}."
  (let ((w (make <gtk-dialog>
             :border-width 6 :has-separator #f :resizable #f
             :title ""))
        (hbox (make <gtk-hbox> :spacing 12 :border-width 6))
        (image (make <gtk-image> :stock (gtk-stock-id type)
                     :icon-size 6))
        (alignment (make <gtk-alignment> :yalign 0.0))
        (vbox (make <gtk-vbox> :spacing 12))
        (plabel (make <gtk-label> :use-markup #t
                      :label (format #f "<b><big>~A</big></b>" primary)
                      :wrap #t :selectable #t))
        (slabel (make <gtk-label> :use-markup #t
                      :label secondary :wrap #t :selectable #t)))
    (set-alignment plabel 0.0 0.0)
    (set-alignment slabel 0.0 0.0)
    (pack-start vbox plabel)
    (pack-end vbox slabel)

    (add alignment image)
    (pack-start hbox alignment #f)
    (pack-end hbox vbox)
    (show-all hbox)
    
    (set (get-vbox w) 'spacing 12)
    (add (get-vbox w) hbox)

    (add-buttons w button-specs)

    (let ((response (run w)))
      (destroy w)
      (genum->symbol (make <gtk-response-type> :value response)))))

(define (question primary secondary . button-specs)
  (apply dialog 'dialog-question primary secondary button-specs))

(define (error-message primary secondary)
  (dialog 'dialog-error primary secondary '(close "Close" close)))
