;; photoblogger
;; Copyright (C) 2004 Andy Wingo

;; This program is free software; you can redistribute it and/or    
;; modify it under the terms of the GNU General Public License as   
;; published by the Free Software Foundation; either version 2 of   
;; the License, or (at your option) any later version.              
;;                                                                  
;; This program is distributed in the hope that it will be useful,  
;; but WITHOUT ANY WARRANTY; without even the implied warranty of   
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    
;; GNU General Public License for more details.                     
;;                                                                  
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, contact:
;;
;; Free Software Foundation           Voice:  +1-617-542-5942
;; 59 Temple Place - Suite 330        Fax:    +1-617-542-2652
;; Boston, MA  02111-1307,  USA       gnu@gnu.org

;;; Commentary:
;;
;;; Code:

(define-module (photoblogger preferences)
  :use-module (photoblogger)
  :use-module (photoblogger state)
  :use-module (gnome gtk)
  :use-module (gnome glade)
  :use-module (gnome gconf)
  :export (show-preferences))

(if (defined? 'preferences)
    (destroy preferences))

(define preferences #f)

(define (make-preferences)
  (let* ((window (get-widget glade-xml "preferences"))
         (width (get-widget glade-xml "width_spinbutton"))
         (height (get-widget glade-xml "height_spinbutton"))
         (remote-path (get-widget glade-xml "remote_path_entry"))
         (web-path (get-widget glade-xml "web_path_entry")))
    (connect window 'delete-event
             (lambda args (hide window) #t))
    (set width 'value (state-ref 'scaled-image-width))
    (connect width 'value-changed
             (lambda args
               (let ((x (inexact->exact (round (get width 'value)))))
                 (if (positive? x) (state-set! 'scaled-image-width x)))))
    (set height 'value (state-ref 'scaled-image-height))
    (connect height 'value-changed
             (lambda args
               (let ((x (inexact->exact (round (get height 'value)))))
                 (if (positive? x) (state-set! 'scaled-image-height x)))))
    (set remote-path 'text (state-ref 'write-url-base))
    (connect remote-path 'changed
             (lambda args
               (state-set! 'write-url-base (get remote-path 'text))))
    (set web-path 'text (state-ref 'http-url-base))
    (connect web-path 'changed
             (lambda args
               (state-set! 'http-url-base (get web-path 'text))))
    (connect (get-widget glade-xml "preferences_close_button") 'clicked
             (lambda args
               (hide preferences)))
    window))

(define (show-preferences)
  (if (not preferences)
      (set! preferences (make-preferences)))
  (present preferences))
